

local TagCompiler = AceLibrary("TagCompiler-1.0")

-----------------------------------------------------------------------------

local pairs, select, type, tonumber = pairs, select, type, tonumber

-----------------------------------------------------------------------------

Assessment.TagData = 
{ 
	["!TagError"] = function(x) Assessment:Print(x) end, 
--	["!TagDebug"] = function(x) Assessment:Debug(x) end 
}

-----------------------------------------------------------------------------

for k,v in pairs(TagCompiler.CommonTagData) do
	Assessment.TagData[k] = v
end

-----------------------------------------------------------------------------

Assessment.TagData.Amount =
{
	tableData = "value.amount",
	numeric = true
}

Assessment.TagData.BarValue =
{
	tableData = "barValue",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.Count =
{
	tableData = "value.count",
	numeric = true
}

Assessment.TagData.FormatDate =
{
	compileString = function(dataTable, prior, format)
						return ("%s%sdate(%s)"):format(prior or "", prior and " and " or "", format)
					end,
	guaranteeResult	= true,
	parameterTypes	= { "string" }
}

Assessment.TagData.FormatTime =
{
	method = function(data, value)
				if type(value) == "number" then
					return date("%H:%M:%S", value)
				end
				return value
			 end,
	noGuaranteeChange = true
}

Assessment.TagData.FormatDuration =
{
	method = function(data, value)
				if type(value) == "number" then
					if value >= 3600 then
						return ("%d:%02d:%02d.%d"):format(value / 3600, value/60%60, value%60, value%1*10)
					elseif value >= 60 then
						return ("%d:%02d.%d"):format(value/60, value%60, value%1*10)
					else
						return ("%.1f"):format(value)
					end
				end
				return value
			 end,
	noGuaranteeChange = true
}

Assessment.TagData.GetVariable =
{
	compileString = function(dataTable, prior, variableName)
						return ("%s%sdata[%s]"):format(prior or "", prior and " and " or "", variableName)
					end,
	parameterTypes	= { "string" }
}

Assessment.TagData.IsAltKeyDown =
{
	compileString = function() return "IsAltKeyDown()" end,
	events = "MODIFIER_STATE_CHANGED",
}

Assessment.TagData.IsControlKeyDown =
{
	compileString = function() return "IsControlKeyDown()" end,
	events = "MODIFIER_STATE_CHANGED",
}

Assessment.TagData.IsPlayerInCombat =
{
	compileString = function() return "UnitAffectingCombat(\"player\")" end,
	events = { "PLAYER_REGEN_ENABLED", "PLAYER_REGEN_DISABLED" }
}

Assessment.TagData.IsReport =
{
	tableData = "isReport",
}

Assessment.TagData.IsShiftKeyDown =
{
	compileString = function() return "IsShiftKeyDown()" end,
	events = "MODIFIER_STATE_CHANGED",
}

Assessment.TagData.Key =
{
	tableData = "key",
	guaranteeResult = true,
}

Assessment.TagData.MaximumAmount =
{
	tableData = "value.maximumAmount",
	numeric = true
}

Assessment.TagData.MaximumBarValue =
{
	tableData = "maximumBarValue",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.Rank =
{
	tableData = "rank",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.SetFightData =
{
	method = function(data, value, fightIndex)
				data.fightData = Assessment:GetFightData(fightIndex)
				return nil
			 end,
	parameterTypes = { "raw" }
}

Assessment.TagData.SetVariable =
{
	method = function(data, value, variableName)
						data[variableName] = value
						return value
					end,
	noGuaranteeChange = true,
	parameterTypes	= { "string" }
}

Assessment.TagData.SortValue =
{
	tableData = "sortValue",
	guaranteeResult = true,
}

Assessment.TagData.TotalBarValue =
{
	tableData = "totalBarValue",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.Value =
{
	tableData = "value",
}

-----------------------------------------------------------------------------

local function CountField(data, ...)
	for i=1,select('#', ...) do
		if type(data) ~= "table" then return 0 end
		local lookup = select(i, ...)
		if lookup == "*" then
			local sum = 0
			for k,v in pairs(data) do
				sum = sum + CountField(v, select(i+1, ...))
			end
			return sum
		else
			data = data[lookup]
		end
	end
	return data and 1 or 0
end

local function GetField(data, ...)
	for i=1,select('#', ...) do
		if type(data) ~= "table" then return nil end
		local v = select(i, ...)
		data = data[tonumber(v) or v]
	end
	return data
end

local function SumField(data, ...)
	for i=1,select('#', ...) do
		if type(data) ~= "table" then return 0 end
		local lookup = select(i, ...)
		if lookup == "*" then
			local sum = 0
			for k,v in pairs(data) do
				sum = sum + SumField(v, select(i+1, ...))
			end
			return sum
		else
			data = data[lookup]
		end
	end
	return data or 0
end

local function SumGroupField(data, groupMembers, ...)
	for i=1,select('#', ...) do
		if type(data) ~= "table" then return 0 end
		local lookup = select(i, ...)
		if lookup == "*" then
			local sum = 0
			for k in pairs(groupMembers) do
				local v = data[k]
				if v then
					sum = sum + SumField(v, select(i+1, ...))
				end
			end
			return sum
		else
			data = data[lookup]
		end
	end
	return data or 0
end

local function MaximumField(data, ...)
	for i=1,select('#', ...) do
		if type(data) ~= "table" then return 0 end
		local lookup = select(i, ...)
		if lookup == "*" then
			local max = 0
			for k,v in pairs(data) do
				local localMax = MaximumField(v, select(i+1, ...))
				if localMax > max then 
					max = localMax
				end
			end
			return max
		else
			data = data[lookup]
		end
	end
	return data or 0
end

local function MaximumGroupField(data, ...)
	for i=1,select('#', ...) do
		if type(data) ~= "table" then return 0 end
		local lookup = select(i, ...)
		if lookup == "*" then
			local max = 0
			for k in pairs(groupMembers) do
				local localMax = MaximumField(k, select(i+1, ...))
				if localMax > max then 
					max = localMax
				end
			end
			return max
		else
			data = data[lookup]
		end
	end
	return data or 0
end

Assessment.TagData.CountField =
{
	method = function(data, value, ...)
				return CountField(data.fightData, ...)
			 end,
	data = "combatTime",
	parameterTypes = { "string", "*string" },
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.CountValueField =
{
	method = function(data, value, ...)
				return CountField(data.value, ...)
			 end,
	parameterTypes = { "string", "*string" },
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.GetField =
{
	method = function(data, value, ...)
				return GetField(data.fightData, ...)
			 end,
	data = "combatTime",
	parameterTypes = { "string", "*string" }
}

Assessment.TagData.GetKeyField =
{
	method = function(data, value, ...)
				return GetField(data.key, ...)
			 end,
	parameterTypes = { "string", "*string" }
}

Assessment.TagData.GetValueField =
{
	compileString = function(dataTable, prior, ...)
						if select('#', ...) == 1 and string.find(select(1, ...), "^\"%w+\"$") then
							return ("%s%sdata.value[%s]"):format(prior or "", prior and " and " or "", select(1, ...))
						else
							dataTable.headerCode["local tagMethodGetValueField = TagCompiler.tagCompilerData.GetValueField.method\n"] = true
							
							local parameterList = select(1, ...)
							for i = 2,select('#', ...) do
								parameterList = parameterList..", "..select(i, ...)
							end
							return ("tagMethodGetValueField(data, %s, %s)"):format(prior or "true", parameterList)
						end
					end,
	method = function(data, value, ...)
				return GetField(data.value, ...)
			 end,
	parameterTypes = { "string", "*string" }
}

Assessment.TagData.SumField =
{
	method = function(data, value, ...)
				return SumField(data.fightData, ...)
			 end,
	data = "combatTime",
	parameterTypes = { "string", "*string" },
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.SumGroupField =
{
	method = function(data, value, ...)
				local fightData = data.fightData
				if not fightData then return 0 end
				local groupMembers = fightData.groupMembers
				if not groupMembers then return 0 end
				return SumGroupField(fightData, groupMembers, ...)
			 end,
	data = "combatTime",
	parameterTypes = { "string", "*string" },
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.SumGroupValueField =
{
	method = function(data, value, ...)
				local fightData = data.fightData
				if not fightData then return 0 end
				local groupMembers = fightData.groupMembers
				if not groupMembers then return 0 end
				return SumGroupField(data.value, groupMembers, ...)
			 end,
	parameterTypes = { "string", "*string" },
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.SumValueField =
{
	method = function(data, value, ...)
				return SumField(data.value, ...)
			 end,
	parameterTypes = { "string", "*string" },
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.MaximumField =
{
	method = function(data, value, ...)
				return MaximumField(data.fightData, ...)
			 end,
	data = "combatTime",
	parameterTypes = { "string", "*string" },
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.MaximumGroupField =
{
	method = function(data, value, ...)
				return MaximumGroupField(data.fightData, ...)
			 end,
	data = "combatTime",
	parameterTypes = { "string", "*string" },
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.MaximumValueField =
{
	method = function(data, value, ...)
				return MaximumField(data.value, ...)
			 end,
	parameterTypes = { "string", "*string" },
	guaranteeResult = true,
	numeric = true
}

-----------------------------------------------------------------------------

local compileCache =
{
	["string"] = { data = { }, events = { }, overrideEvents = { }, tag = "", method = function() return "" end },
}

function Assessment:CompileTag(tag, resultType)
	tag = tag or ""
	local cacheTag = resultType..tag
	if compileCache[cacheTag] then
		return compileCache[cacheTag]
	end

	local result = TagCompiler:Compile(tag, self.TagData, resultType)
	compileCache[cacheTag] = result
	return result
end

function Assessment:FixTagCasing(tag, resultType)
	return TagCompiler:FixCasing(tag:gsub("|c%x%x%x%x%x%x%x%x", ""):gsub("|r", ""), self.TagData, resultType)
end

function Assessment:HighlightTagSyntax(tag, resultType)
	return TagCompiler:HighlightSyntax(tag, self.TagData, resultType)
end

-----------------------------------------------------------------------------
